#ifndef _FOODDELIVERYBASE_H_
#define _FOODDELIVERYBASE_H_
#include "Arduino.h"
/*wifi*/
#include "WiFi.h"
/*mqtt*/
#include "AsyncMqttClient.h"

/*mavlink*/
#include "../lib/mavlink/ardupilotmega/mavlink.h"
#include "../lib/mavlink/common/mavlink.h"
/*json库*/
#include "ArduinoJson.h"
/*异步库*/
#include "Ticker.h"
/*udp发送*/
#include "WiFiUdp.h"

#define MAVLINK_SYSTEM_ID 0xBE    // 飞控系统 ID
#define MAVLINK_COMPONENT_ID 0xBE // 飞控组件 ID

class FoodCube
{
public:
  /*飞行航点任务相关属性*/
  bool writeState = false;    // 是否是写入状态
  int8_t writeSeq = -1;       // 飞控反馈 需写入航点序列号
  int8_t futureSeq = 0;       // 记录将来要写入 航点序列号
  int8_t missionArkType = -1; // 航点写入是否成功
  /*是否处于 加速度计校准状态*/
  boolean isAnalyzeAcce = false; // 在接收开始校准时更改为true 这个状态匹配文本信息判断 校准步骤给前端
  /*航点任务 送餐信息喊话*/
  String questVoiceStr;
  /*前端模拟遥控的油门通道*/
  uint16_t channels[4] = {1500, 1500, 1500, 1500};
  /*初始化*/
  FoodCube(const char *userSsid, const char *userPassword, const char *userMqttServer, int userMqttPort, const char *userMqttName, const char *userMqttPassword, String userMqttTopic, uint8_t userMavlinkSerial, uint8_t userVoiceSerial, const char *userUdpServerIP, uint32_t userUdpServerPort);
  ~FoodCube();
  /*日志打印*/
  void log(String val);
  void log(const char *val);
  void log(int val);
  void log(IPAddress val);
  void log(bool val);
  void logln(String val);
  void logln(const char *val);
  void logln(int val);
  void logln(IPAddress val);
  void logln(bool val);
  /*get set value*/
  String getMacAdd();
  /*wifi*/
  String macAdd; // 板子的物理地址(已去掉冒号分隔符)
  void connectWifi();
  bool checkWiFiStatus();
  /*mqtt*/
  String mqttTopic;            // 订阅mqtt主题
  AsyncMqttClient *mqttClient; // 指向 mqtt服务器连接 对象
  void connectMqtt(String topicSub);
  void mqttLoop(String topicSub);
  void subscribeTopic(String topicString, int Qos);
  void pubMQTTmsg(String topicString, String messageString, uint8_t qos);

  /*声音模块控制*/
  // 枚举，音量
  enum VoiceVolume
  {
    V1 = 1,
    V2 = 2,
    V3 = 3,
    V4 = 4,
    V5 = 5,
    V6 = 6,
    V7 = 7,
    V8 = 8,
    V9 = 9
  };
  unsigned long lastRunTime = 0; // playText()上次运行时间戳
  void playText(String str, VoiceVolume vol = V1);
  void SetplayvolMax();
  uint8_t chekVoiceMcu();
  void stopVoice();
  /*mavlink*/
  String setNBit(String str, uint8_t n, uint8_t i);
  void mav_request_data();
  void mav_parameter_data(const char *param_id);
  void mav_request_home_position();
  void comm_receive(void (*pFun)(uint8_t));
  void mav_mission_count(uint8_t taskcount);
  void mav_mission_item(int8_t seq, uint8_t frame, uint8_t command, uint8_t current, uint8_t autocontinue, double param1, double param2, double param3, double param4, double x, double y, double z);
  void mav_channels_override(uint16_t chan[]);
  void mav_send_text(uint8_t severity, const char *text);

  void mav_send_command(mavlink_command_long_t &msg_cmd);
  void sendCommandAck(uint16_t command, uint8_t result);
  void setParam(const char *param_id, float value);
  void sendSetPositionTargetGlobalInt(double lat_deg, double lon_deg, double alt_m);

  /*云台相机控制*/
  void udpSendToCamera(uint8_t *p_command, uint32_t len);
  void Camera_action_down();
  void Camera_action_ret();
  void Camera_action_move(uint8_t vpitch, uint8_t vyaw);
  void Camera_action_zoom(uint8_t vzoom);

private:
  const char *ssid;         // wifi帐号
  const char *password;     // wifi密码
  const char *mqttServer;   // mqtt服务器地址
  int mqttPort;             // mqtt服务器端口
  const char *mqttName;     // mqtt帐号
  const char *mqttPassword; // mqtt密码

  uint8_t mavlinkSerial; // 飞控占用的串口号
  uint8_t voiceSerial;   // 飞控占用的串口号

  WiFiClient wifiClient; // 网络客户端
  IPAddress localIp;     // 板子的IP地址

  bool wificonnected; // 网络是否连接
  /*云台相机控制*/
  WiFiUDP udp;                   // udp信息操作对象
  const char *udpServerIP;       // 云台相机ip地址
  uint32_t udpServerPort;        // 云台相机端口
  unsigned long _tm_mqttconnect; // mqtt上次连接时间

  // 摄像头控制 校验代码
  const uint16_t crc16_tab[256] = {0x0, 0x1021, 0x2042, 0x3063, 0x4084, 0x50a5, 0x60c6, 0x70e7,
                                   0x8108, 0x9129, 0xa14a, 0xb16b, 0xc18c, 0xd1ad, 0xe1ce, 0xf1ef,
                                   0x1231, 0x210, 0x3273, 0x2252, 0x52b5, 0x4294, 0x72f7, 0x62d6,
                                   0x9339, 0x8318, 0xb37b, 0xa35a, 0xd3bd, 0xc39c, 0xf3ff, 0xe3de,
                                   0x2462, 0x3443, 0x420, 0x1401, 0x64e6, 0x74c7, 0x44a4, 0x5485,
                                   0xa56a, 0xb54b, 0x8528, 0x9509, 0xe5ee, 0xf5cf, 0xc5ac, 0xd58d,
                                   0x3653, 0x2672, 0x1611, 0x630, 0x76d7, 0x66f6, 0x5695, 0x46b4,
                                   0xb75b, 0xa77a, 0x9719, 0x8738, 0xf7df, 0xe7fe, 0xd79d, 0xc7bc,
                                   0x48c4, 0x58e5, 0x6886, 0x78a7, 0x840, 0x1861, 0x2802, 0x3823,
                                   0xc9cc, 0xd9ed, 0xe98e, 0xf9af, 0x8948, 0x9969, 0xa90a, 0xb92b,
                                   0x5af5, 0x4ad4, 0x7ab7, 0x6a96, 0x1a71, 0xa50, 0x3a33, 0x2a12,
                                   0xdbfd, 0xcbdc, 0xfbbf, 0xeb9e, 0x9b79, 0x8b58, 0xbb3b, 0xab1a,
                                   0x6ca6, 0x7c87, 0x4ce4, 0x5cc5, 0x2c22, 0x3c03, 0xc60, 0x1c41,
                                   0xedae, 0xfd8f, 0xcdec, 0xddcd, 0xad2a, 0xbd0b, 0x8d68, 0x9d49,
                                   0x7e97, 0x6eb6, 0x5ed5, 0x4ef4, 0x3e13, 0x2e32, 0x1e51, 0xe70,
                                   0xff9f, 0xefbe, 0xdfdd, 0xcffc, 0xbf1b, 0xaf3a, 0x9f59, 0x8f78,
                                   0x9188, 0x81a9, 0xb1ca, 0xa1eb, 0xd10c, 0xc12d, 0xf14e, 0xe16f,
                                   0x1080, 0xa1, 0x30c2, 0x20e3, 0x5004, 0x4025, 0x7046, 0x6067,
                                   0x83b9, 0x9398, 0xa3fb, 0xb3da, 0xc33d, 0xd31c, 0xe37f, 0xf35e,
                                   0x2b1, 0x1290, 0x22f3, 0x32d2, 0x4235, 0x5214, 0x6277, 0x7256,
                                   0xb5ea, 0xa5cb, 0x95a8, 0x8589, 0xf56e, 0xe54f, 0xd52c, 0xc50d,
                                   0x34e2, 0x24c3, 0x14a0, 0x481, 0x7466, 0x6447, 0x5424, 0x4405,
                                   0xa7db, 0xb7fa, 0x8799, 0x97b8, 0xe75f, 0xf77e, 0xc71d, 0xd73c,
                                   0x26d3, 0x36f2, 0x691, 0x16b0, 0x6657, 0x7676, 0x4615, 0x5634,
                                   0xd94c, 0xc96d, 0xf90e, 0xe92f, 0x99c8, 0x89e9, 0xb98a, 0xa9ab,
                                   0x5844, 0x4865, 0x7806, 0x6827, 0x18c0, 0x8e1, 0x3882, 0x28a3,
                                   0xcb7d, 0xdb5c, 0xeb3f, 0xfb1e, 0x8bf9, 0x9bd8, 0xabbb, 0xbb9a,
                                   0x4a75, 0x5a54, 0x6a37, 0x7a16, 0xaf1, 0x1ad0, 0x2ab3, 0x3a92,
                                   0xfd2e, 0xed0f, 0xdd6c, 0xcd4d, 0xbdaa, 0xad8b, 0x9de8, 0x8dc9,
                                   0x7c26, 0x6c07, 0x5c64, 0x4c45, 0x3ca2, 0x2c83, 0x1ce0, 0xcc1,
                                   0xef1f, 0xff3e, 0xcf5d, 0xdf7c, 0xaf9b, 0xbfba, 0x8fd9, 0x9ff8,
                                   0x6e17, 0x7e36, 0x4e55, 0x5e74, 0x2e93, 0x3eb2, 0xed1, 0x1ef0};
  uint16_t CRC16_cal(uint8_t *ptr, uint32_t len, uint16_t crc_init);
  void crc_check_16bites(uint8_t *pbuf, uint32_t len, uint16_t *p_result);

  /*串口输出*/
  void SWrite(uint8_t buf[], int len, uint8_t swSerial);
};

#endif